@@ -1,4 +1,4 @@
-import fsbc.Paths
+import fsbc.paths
 import nose.tools
 
 
@@ -11,5 +11,5 @@ def test_mypy():
 
 def test_doctest():
     import doctest
-    failure_count, test_count = doctest.testmod(fsbc.Paths)
+    failure_count, test_count = doctest.testmod(fsbc.paths)
     nose.tools.assert_equals(failure_count, 0)
