@@ -2,10 +2,10 @@
 from fsgs.context import fsgs
 import os
 import json
-from fsbc.Paths import Paths
+from fsbc.paths import Paths
 from fsgs.amiga.Amiga import Amiga
 from fsgs.FileDatabase import FileDatabase
-from fsgs.ogd.client import OGDClient
+from fsgs.network import openretro_url_prefix
 
 
 class ValueConfigLoader(object):
@@ -19,9 +19,10 @@ class ValueConfigLoader(object):
         self.values = {}
         self.uuid = uuid
         self._file_list = None
+        self._cue_sheets = None
         if uuid:
-            self.options["database_url"] = "http://{0}/game/{1}".format(
-                OGDClient.get_server(), uuid)
+            self.options["database_url"] = "{0}/game/{1}".format(
+                openretro_url_prefix(), uuid)
 
     def get_config(self):
         return self.config.copy()
@@ -45,13 +46,14 @@ class ValueConfigLoader(object):
         platform = self.values.get("platform", "Amiga")
         self.options["platform"] = platform.lower()
 
-        if platform == "Amiga":
+        platform = platform.lower()
+        if platform == "amiga":
             pass
-        elif platform == "CD32":
+        elif platform == "cd32":
             amiga_model = "CD32"
             cd_based = True
             self.options["joystick_port_1_mode"] = "cd32 gamepad"
-        elif platform == "CDTV":
+        elif platform == "cdtv":
             amiga_model = "CDTV"
             cd_based = True
         else:
@@ -114,6 +116,11 @@ class ValueConfigLoader(object):
         for key, value in self.options.items():
             self.config[key] = value
 
+        if (self.config.get("amiga_model", "") == "A500" and
+                self.config.get("slow_memory") == "0"):
+            self.config["amiga_model"] = "A500/512K"
+            self.config["slow_memory"] = ""
+
         self.set_name_and_uuid()
         self.contract_paths()
         return self.get_config()
@@ -204,8 +211,8 @@ class ValueConfigLoader(object):
             self.options["save_disk"] = value
         elif key == "whdload_version":
             self.options["x_whdload_version"] = value
-        elif key == "whdload_icon":
-            self.options["x_whdload_icon"] = value
+        # elif key == "whdload_icon":
+        #     self.options["__whdload_icon"] = value
         elif key == "kickstart":
             if value == "1.2":
                 self.options["amiga_model"] = "A1000"
@@ -281,7 +288,7 @@ class ValueConfigLoader(object):
                 "joystick_port_4_mode"]:
             self. load_joystick_port_x_mode_option(key, value)
         elif key in [
-                "amiga_model", "accuracy",
+                "amiga_model", "accuracy", "cdrom_drive_0_delay",
                 "floppy_drive_count", "slow_memory", "front_sha1",
                 "screen1_sha1", "screen2_sha1", "screen3_sha1",
                 "screen4_sha1", "screen5_sha1", "title_sha1",
@@ -337,6 +344,12 @@ class ValueConfigLoader(object):
             self._file_list = json.loads(file_list_json)
         return self._file_list
 
+    def get_cue_sheets(self):
+        if self._cue_sheets is None:
+            cue_sheets_json = self.values.get("cue_sheets", "[]")
+            self._cue_sheets = json.loads(cue_sheets_json)
+        return self._cue_sheets
+
     def build_media_list(self, floppies=False, cds=False, hds=False):
         media_list = []
         added = set()
@@ -437,7 +450,31 @@ class ValueConfigLoader(object):
         raise Exception("cdrom_sha1_to_uri: could not find " + sha1)
 
     def load_cdroms(self):
-        media_list = self.build_media_list(cds=True)
+        # media_list = self.build_media_list(cds=True)
+        cue_sheets = self.get_cue_sheets()
+        if cue_sheets:
+            return self.load_cdroms_from_cue_sheets(cue_sheets)
+        cue = False
+        ccd = False
+        # iso = False
+        for file_item in self.get_file_list():
+            if file_item["name"].endswith(".cue"):
+                cue = True
+            if file_item["name"].endswith(".ccd"):
+                ccd = True
+            # if file_item["name"].endswith(".iso"):
+            #     iso = True
+        media_list = []
+        if ccd:
+            ext = ".ccd"
+        elif cue:
+            ext = ".cue"
+        else:
+            ext = ".iso"
+        for file_item in self.get_file_list():
+            if file_item["name"].endswith(ext):
+                media_list.append((file_item["name"], file_item["sha1"]))
+        print("load_cdroms media_list =", media_list)
         for i, values in enumerate(media_list):
             path, sha1 = values
             path = self.cdrom_sha1_to_uri(sha1)
@@ -447,6 +484,13 @@ class ValueConfigLoader(object):
             self.config["cdrom_image_{0}".format(i)] = path
             self.config["x_cdrom_image_{0}_sha1".format(i)] = sha1
 
+    def load_cdroms_from_cue_sheets(self, cue_sheets):
+        for i, item in enumerate(cue_sheets):
+            path = "game://{}/{}".format(self.uuid, item["name"])
+            if i < 1:
+                self.config["cdrom_drive_{0}".format(i)] = path
+            self.config["cdrom_image_{0}".format(i)] = path
+
     def load_hard_drives(self):
         media_list = self.build_media_list(hds=True)
         print(media_list)
