@@ -1,14 +1,13 @@
 from fsgs.drivers.dos import DOSDriver
 from fsgs.platform import PlatformHandler
-from .loader import SimpleLoader
+from fsgs.platforms.loader import SimpleLoader
 
 
 class DOSPlatformHandler(PlatformHandler):
-
     PLATFORM_NAME = "DOS"
 
     def __init__(self):
-        PlatformHandler.__init__(self)
+        super().__init__()
 
     def get_loader(self, fsgs):
         return DOSLoader(fsgs)
@@ -18,10 +17,9 @@ class DOSPlatformHandler(PlatformHandler):
 
 
 class DOSLoader(SimpleLoader):
-
     def load_files(self, values):
         self.config["file_list"] = values["file_list"]
 
     def load_extra(self, values):
-        self.config["hd_startup"] = values["hd_startup"]
-        self.config["dosbox_cpu_cycles"] = values["dosbox_cpu_cycles"]
+        for key in ["hd_startup", "dosbox_cpu_cycles", "sblaster_irq"]:
+            self.config[key] = values[key]
