@@ -1,7 +1,11 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #if !defined(USE_OPENAL)
 
-#include <stdint.h>
 #include <fs/emu.h>
+#include <stdint.h>
 
 void fs_emu_init_audio_stream(int stream,
         fs_emu_audio_stream_options *options) {
@@ -56,4 +60,8 @@ void fs_emu_audio_render_debug_info(uint32_t *texture) {
 
 }
 
-#endif
\ No newline at end of file
+#else
+
+int libfsemu_audio_dummy_dummy;
+
+#endif
