@@ -14,13 +14,13 @@
 
 #ifdef HAVE_AL_AL_H
 #include <AL/al.h>
-#else
-#include <al.h>
+#elif defined(HAVE_OPENAL_AL_H)
+#include <OpenAL/al.h>
 #endif
 #ifdef HAVE_AL_ALC_H
 #include <AL/alc.h>
-#else
-#include <alc.h>
+#elif defined(HAVE_OPENAL_ALC_H)
+#include <OpenAL/alc.h>
 #endif
 
 #include "libfsemu.h"
@@ -458,22 +458,26 @@ void fs_emu_audio_video_sync(int time_ms)
     }
 }
 
-static void log_openal_info()
+static void log_openal_info(void)
 {
+    fs_log("[OPENAL] Information:\n");
     if (alGetString(AL_VERSION)) {
-        fs_log("OPENAL: Version \"%s\"\n", alGetString(AL_VERSION));
+        fs_log("[OPENAL] Version \"%s\"\n", alGetString(AL_VERSION));
     }
     if (alGetString(AL_RENDERER)) {
-        fs_log("OPENAL: Renderer \"%s\"\n", alGetString(AL_RENDERER));
+        fs_log("[OPENAL] Renderer \"%s\"\n", alGetString(AL_RENDERER));
     }
     if (alGetString(AL_VENDOR)) {
-        fs_log("OPENAL: Vendor \"%s\"\n", alGetString(AL_VENDOR));
+        fs_log("[OPENAL] Vendor \"%s\"\n", alGetString(AL_VENDOR));
     }
     if (alGetString(AL_EXTENSIONS)) {
-        fs_log("OPENAL: Extensions \"%s\"\n", alGetString(AL_EXTENSIONS));
+        fs_log("[OPENAL] Extensions \"%s\"\n", alGetString(AL_EXTENSIONS));
     }
+}
 
-    fs_log("openal devices:\n");
+static void log_openal_devices(void)
+{
+    fs_log("[OPENAL] Devices:\n");
     if (alcIsExtensionPresent(NULL, "ALC_ENUMERATION_EXT") == AL_TRUE) {
         const char *s = (const char *) alcGetString(NULL,
                                                     ALC_DEVICE_SPECIFIER);
@@ -486,8 +490,7 @@ static void log_openal_info()
     } else {
         fs_log(" - no support for device enumeration\n");
     }
-
-    fs_log("OPENAL: Default device: %s\n",
+    fs_log("[OPENAL] Default device: %s\n",
            alcGetString(NULL, ALC_DEFAULT_DEVICE_SPECIFIER));
 }
 
@@ -507,22 +510,22 @@ static void openal_audio_init()
     // select the "preferred device"
     g_device = alcOpenDevice(NULL);
     if (g_device) {
-        fs_log("OPENAL: Opened device: %s\n",
+        fs_log("[OPENAL] Opened device: %s\n",
                alcGetString(g_device, ALC_DEVICE_SPECIFIER));
     } else {
-        fs_log("OPENAL: alcOpenDevice returned NULL\n");
-        fs_emu_warning("OPENAL: Could not open audio device device\n");
+        fs_log("[OPENAL] NULL from alcOpenDevice\n");
         ALenum error_code = alGetError();
-        fs_log("OPENAL: error code %d\n", error_code);
+        fs_log("[OPENAL] Error code %d\n", error_code);
         if (alGetString(error_code)) {
-            fs_log("(%s)\n", alGetString(error_code));
+            fs_log("[OPENAL] %s\n", alGetString(error_code));
         }
+        fs_emu_warning("OPENAL: Could not open audio device");
     }
-
-    log_openal_info();
     if (!g_device) {
         return;
     }
+    log_openal_info();
+    log_openal_devices();
 
     int frequencies[] = { 48000, 44100, 0 };
     if (fs_config_get_int("audio_frequency") != FS_CONFIG_NONE) {
@@ -615,20 +618,26 @@ static void openal_audio_init()
 
 void fs_emu_audio_init()
 {
-    fs_log("fs_emu_audio_init\n");
+    fs_log("OPENAL: fs_emu_audio_init\n");
     openal_audio_init();
 }
 
 static void openal_audio_shutdown()
 {
-    //alDeleteSources(NUM_SOURCES, source);
-    //alDeleteBuffers(NUM_BUFFERS, buffers);
+    for (int i = 0; i < MAX_STREAMS; i++) {
+        if (g_streams[i]) {
+            fs_log("OPENAL: Stopping stream %d\n", i);
+            alSourceStop(g_streams[i]->source);
+        }
+    }
     alcMakeContextCurrent(NULL);
-    if (!g_context) {
+    if (g_context) {
+        fs_log("OPENAL: alcDestroyContext\n");
         alcDestroyContext(g_context);
         g_context = NULL;
     }
     if (g_device) {
+        fs_log("OPENAL: alcCloseDevice\n");
         alcCloseDevice(g_device);
         g_device = NULL;
     }
@@ -636,6 +645,7 @@ static void openal_audio_shutdown()
 
 void fs_emu_audio_shutdown()
 {
+    fs_log("OPENAL: fs_emu_audio_shutdown\n");
     openal_audio_shutdown();
 }
 
