@@ -41,7 +41,7 @@ void fs_emu_clear_menu_input_states(int key);
 void fs_emu_audio_set_default_pitch(double pitch);
 void fs_emu_initialize_opengl();
 
-void fs_emu_volume_control();
+void fs_emu_volume_control(int volume);
 
 void fs_emu_init_render();
 
