@@ -17,6 +17,10 @@ void fs_init()
 {
 #ifdef USE_GLIB
 #ifdef USE_REAL_MALLOC
+#if !GLIB_CHECK_VERSION (2, 46, 0)
+    /* g_mem_is_system_malloc has been deprecated since version 2.46 and
+     * should not be used in newly-written code. GLib always uses the system
+     * malloc. */
     GMemVTable vtable;
     memset(&vtable, 0, sizeof(GMemVTable));
     vtable.malloc = malloc;
@@ -24,6 +28,7 @@ void fs_init()
     vtable.free = free;
     g_mem_set_vtable(&vtable);
 #endif
+#endif
 #if !GLIB_CHECK_VERSION (2, 32, 0)
     /* g_thread_init is deprecated since 2.32, and is only needed for older
      * versions of Glib (quite a bit older than 2.32, actually) */
