@@ -35,7 +35,6 @@ int g_fs_ml_video_sync = 0;
 int g_fs_ml_video_sync_low_latency = 0;
 int g_fs_ml_vblank_sync = 0;
 
-int g_fs_ml_target_refresh_rate = 0;
 int g_fs_ml_target_frame_time = 0;
 
 // when OpenGL is reinitialized, we update this version (because we may need
@@ -166,78 +165,8 @@ void fs_ml_video_set_post_render_function(fs_ml_void_function function) {
     g_fs_ml_video_post_render_function = function;
 }
 
-int fs_ml_handle_keyboard_shortcut(fs_ml_event *event) {
-    int state = event->key.state;
-    int key = event->key.keysym.sym;
-    int mod = event->key.keysym.mod;
-    //printf("%d %d -- %d %d\n", key, mod, SDLK_RETURN, KMOD_ALT);
-    int special = mod & (FS_ML_KEY_MOD_F11 | FS_ML_KEY_MOD_F12);
-
-#ifdef MACOSX
-    int alt_mod = mod & (FS_ML_KEY_MOD_ALT | FS_ML_KEY_MOD_META);
-#else
-    int alt_mod = mod & FS_ML_KEY_MOD_ALT;
-#endif
-
-    if (special) {
-        if (key == FS_ML_KEY_F) {
-            if (state) {
-                fs_ml_toggle_fullscreen();
-            }
-            //return 1;
-        }
-        else if (key == FS_ML_KEY_Q) {
-            if (state) {
-                fs_ml_quit();
-            }
-            //return 1;
-        }
-    }
-
-    if (key == FS_ML_KEY_RETURN && alt_mod) {
-        if (state) {
-            fs_log("ALT+Return key press detected\n");
-            fs_ml_toggle_fullscreen();
-        }
-        return 1;
-    }
-    else if (key == FS_ML_KEY_F4 && alt_mod) {
-        if (state) {
-            fs_log("ALT+F4 key press detected\n");
-            fs_ml_quit();
-        }
-    }
-    else if (key == FS_ML_KEY_TAB && alt_mod) {
-        if (state) {
-            fs_log("ALT+Tab key press detected\n");
-#ifdef USE_SDL2
-
-#else
-#ifdef WINDOWS
-            // input grab will be released be deactivation
-                        // event in this case
-#else
-            if (fs_ml_has_input_grab()) {
-                fs_log("- releasing input grab");
-                fs_ml_grab_input(0, 1);
-                g_fs_ml_had_input_grab = 1;
-            }
-            if (g_fs_emu_video_fullscreen == 1 &&
-                    g_fs_emu_video_fullscreen_mode == 0) {
-                fs_log("- switching to window mode\n");
-                g_fs_ml_was_fullscreen = 1;
-                fs_ml_toggle_fullscreen();
-            }
-#endif
-#endif
-        }
-        return 1;
-    }
-
-    return 0;
-}
-
-void fs_ml_init() {
+void fs_ml_init()
+{
     fs_log("fs_ml_init (operating system: %s)\n", OS_NAME);
 
     g_fs_ml_video_render_function = NULL;
@@ -286,27 +215,14 @@ void fs_ml_init() {
 #endif
 }
 
-void fs_ml_init_2() {
-    fs_ml_video_mode mode;
-    memset(&mode, 0, sizeof(fs_ml_video_mode));
-    if (fs_ml_video_mode_get_current(&mode) == 0) {
-        g_fs_ml_target_refresh_rate = mode.fps;
-    }
-    else {
-        fs_log("WARNING: could not read refresh rate from current mode\n");
-        g_fs_ml_target_refresh_rate = 0;
-    }
-    if (g_fs_ml_target_refresh_rate) {
-        g_fs_ml_target_frame_time = 1000000 / g_fs_ml_target_refresh_rate;
-    }
-    fs_log("assuming refresh rate: %d (%d usec per frame)\n",
-            g_fs_ml_target_refresh_rate, g_fs_ml_target_frame_time);
-
+void fs_ml_init_2(void)
+{
     fs_ml_input_init();
 
     g_fs_ml_video_screenshot_mutex = fs_mutex_create();
 }
 
-double fs_ml_get_refresh_rate() {
-    return g_fs_ml_target_refresh_rate;
+double fs_ml_get_refresh_rate(void)
+{
+    return g_fs_emu_video_frame_rate_host;
 }
