@@ -11,6 +11,8 @@
 #include "sysconfig.h"
 #include "sysdeps.h"
 
+#ifdef ARCADIA
+
 #include "options.h"
 #include "uae.h"
 #include "uae/memory.h"
@@ -328,7 +330,7 @@ static addrbank arcadia_boot_bank = {
 	arbb_lget, arbb_wget, arbb_bget,
 	arbb_lput, arbb_wput, arbb_bput,
 	arbb_xlate, arbb_check, NULL, NULL, _T("Arcadia BIOS"),
-	arbb_lget, arbb_wget, ABFLAG_ROM | ABFLAG_SAFE,
+	arbb_lget, arbb_wget, ABFLAG_ROM | ABFLAG_SAFE, S_READ, S_WRITE,
 	NULL, arbb_mask
 };
 
@@ -408,7 +410,7 @@ static addrbank arcadia_rom_bank = {
 	arb_lget, arb_wget, arb_bget,
 	arb_lput, arb_wput, arb_bput,
 	arb_xlate, arb_check, NULL, NULL, _T("Arcadia Game ROM"),
-	arb_lget, arb_wget, ABFLAG_ROM | ABFLAG_SAFE,
+	arb_lget, arb_wget, ABFLAG_ROM | ABFLAG_SAFE, S_READ, S_WRITE,
 	NULL, arb_mask
 };
 
@@ -545,3 +547,5 @@ struct romdata *scan_arcadia_rom (TCHAR *path, int cnt)
 	}
 	return rd;
 }
+
+#endif /* ARCADIA */
