@@ -14,7 +14,11 @@
 #include "sysconfig.h"
 #include "sysdeps.h"
 
+#ifdef HAVE_SYS_TIMEB_H
 #include <sys/timeb.h>
+#else
+
+#endif
 
 #include "options.h"
 #include "blkdev.h"
@@ -480,12 +484,18 @@ static void *cdda_play_func (void *v)
 		if (oldplay != cdu->cdda_play) {
 			struct cdtoc *t;
 			int sector, diff;
+#ifdef HAVE_SYS_TIMEB_H
 			struct _timeb tb1, tb2;
+#else
+#warning Missing timing functions
+#endif
 
 			idleframes = 0;
 			silentframes = 0;
 			foundsub = false;
+#ifdef HAVE_SYS_TIMEB_H
 			_ftime (&tb1);
+#endif
 			cdda_pos = cdu->cdda_start;
 			oldplay = cdu->cdda_play;
 			sector = cdu->cd_last_pos = cdda_pos;
@@ -539,9 +549,13 @@ static void *cdda_play_func (void *v)
 			}
 			cdda_pos -= idleframes;
 
+#ifdef HAVE_SYS_TIMEB_H
 			_ftime (&tb2);
 			diff = (tb2.time * (uae_s64)1000 + tb2.millitm) - (tb1.time * (uae_s64)1000 + tb1.millitm);
 			diff -= cdu->cdda_delay;
+#else
+			diff = 0;
+#endif
 			if (idleframes >= 0 && diff < 0 && cdu->cdda_play > 0)
 				sleep_millis(-diff);
 			setstate (cdu, AUDIO_STATUS_IN_PROGRESS);
