@@ -1,9 +1,7 @@
 #ifndef UAE_PARALLEL_H
 #define UAE_PARALLEL_H
 
-#ifdef FSUAE // NL
 #include "uae/types.h"
-#endif
 
 extern int isprinter (void);
 extern void doprinter (uae_u8);
@@ -12,4 +10,10 @@ extern void closeprinter (void);
 extern int isprinteropen (void);
 extern void initparallel (void);
 
-#endif // UAE_PARALLEL_H
+void parallel_exit(void);
+#ifdef FSUAE
+void parallel_ack(void);
+void parallel_poll_ack(void);
+#endif
+
+#endif /* UAE_PARALLEL_H */
