@@ -15,6 +15,7 @@ void romlist_patch_rom(uae_u8 *buf, size_t size);
 void keyboard_settrans (void);
 
 extern int g_amiga_savestate_docompress;
+extern bool g_fs_uae_jit_compiler;
 
 #include <stdio.h>
 #include <stdint.h>
@@ -56,4 +57,4 @@ extern int g_uae_min_first_line_ntsc;
 
 void od_fs_update_leds(void);
 
-#endif // UAE_FS_H
+#endif /* UAE_FS_H */
