@@ -2,11 +2,10 @@
 #include <stdio.h>
 #include <stdarg.h>
 
-//UAEAPI void UAECALL uae_log(const char *format, ...)
 void UAECALL uae_log(const char *format, ...)
 {
-	/* temp hack to be able to use UAE_LOG_VA_ARGS_FULL without
-	 * getting a circular dependency */
+	/* Redirect UAE_LOG_VA_ARGS_FULL to use write_log instead */
 #define uae_log write_log
 	UAE_LOG_VA_ARGS_FULL(format);
+#undef uae_log
 }
